/***************************************************************************
 *  Program Name      : getcal.c
 *  Type              : Subroutine
 *  Function          : Program extracts earth view of each spot and 
 *			calibration coefficients from AMSU-A 1B* data,
 *			and call subroutines to convert counts to radiances 
 *			and then radiances to antenna temperatures
 *  Input Files       : None 
 *  Output Files      : None
 *  Subroutine Called : rad2at.c, apc.c, leapyear.c 
 *  Called by         : gnrt_fcdr.c
 *
 *  Modification History:
 *      Date       Version         Description of Changes
 *     ------     ---------       ------------------------
 *   8/30/2000      v2.0
 *************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"

/*****************************************************************/
//void cnt2rd();
void rad2at();
void apc();
long int leapyear();

/*****************************************************************/
void getcal(short int iscan, short int numscan)
{
   short int   	ichan, ispot;
   int         	insnum, numchan, numspot;
   int 	       	a2ch[NUMCHAN_A] = {1,2,3,15}; 
   float	countc_bar, countw_bar;

   long int	lyflag;
   float	dyear,days; 
   float        Rl_temp, Z_temp;

   numchan = NUMCHAN_A;
   numspot = NUMSPOT_A;
   insnum  = 2;

   lyflag = leapyear(geo_scantime[1].year);
   if (lyflag == 1)
      days = 366;
   else
      days = 365;
   dyear = geo_scantime[iscan].doy;
   dyear += geo_scantime[iscan].hour / 24;
   dyear = geo_scantime[iscan].year + dyear / days - Year0;
 
/*---------------------------------------------------------*
 * Extract earth views (counts) and calibration coeffs
 *---------------------------------------------------------*/
   for(ispot = 0; ispot < NUMSPOT_A; ispot++)
   {
     for(ichan = 0; ichan < NUMCHAN_A - 1; ichan++)
     {
       if(ichan < 2)
         count[ichan][ispot] = scanline[iscan].scene_telemetry_a2[ispot*4+ichan+2];
       else
         count[ichan][ispot] = scanline[iscan].scene_telemetry_a1[ispot*17+ichan+2];
     }       

     ichan = 3;
     count[ichan][ispot] = scanline[iscan].scene_telemetry_a1[ispot*17+ichan+2+11];	
   }

// processing of countc and countw has been moved to rama_scn.c
     for(ichan = 0; ichan < 2; ichan++)
     {
       countc[ichan][0] = scanline[iscan].cold_cal_telemetry_a2[ichan+2] - scanline[iscan].countc_corr[a2ch[ichan]-1];
       countc[ichan][1] = scanline[iscan].cold_cal_telemetry_a2[ichan+4] - scanline[iscan].countc_corr[a2ch[ichan]-1];
       countw[ichan][0] = scanline[iscan].warm_cal_telemetry_a2[ichan+2];
       countw[ichan][1] = scanline[iscan].warm_cal_telemetry_a2[ichan+4];
     }

     ichan = 2;
     countc[ichan][0] = scanline[iscan].cold_cal_telemetry_a1[ichan+2] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countc[ichan][1] = scanline[iscan].cold_cal_telemetry_a1[ichan+15] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countw[ichan][0] = scanline[iscan].warm_cal_telemetry_a1[ichan+2];
     countw[ichan][1] = scanline[iscan].warm_cal_telemetry_a1[ichan+15];

     ichan = 3;
     countc[ichan][0] = scanline[iscan].cold_cal_telemetry_a1[ichan+2+11] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countc[ichan][1] = scanline[iscan].cold_cal_telemetry_a1[ichan+15+11] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countw[ichan][0] = scanline[iscan].warm_cal_telemetry_a1[ichan+2+11];
     countw[ichan][1] = scanline[iscan].warm_cal_telemetry_a1[ichan+15+11];

/*---------------------------------------------------------*
 * Calculate radiances from counts
 *---------------------------------------------------------*/
   //cnt2rd(numchan, numspot);
   //printf("rad is %f, radc is %f, radw is %f.\n", calrad[0][0], calradc[0][0], calradw[0][0]);

/*---------------------------------------------------------*
 * Perform Antenna Pattern Correction
 *---------------------------------------------------------*/
//   apc(numchan, numspot);

/*---------------------------------------------------------*
 * Convert radiances to antenna temperatures
 *---------------------------------------------------------*/
/*---------------------------------------------------------*
 * Calculate S, Rl, and Z
 *---------------------------------------------------------*/

   for(ichan = 0; ichan < NUMCHAN_A; ichan++)
   {
      if (scanneed[iscan] == 7)
      {
          countc_bar = cc[iscan][ichan];
          countc_bar += (cc[iscan-1][ichan] + cc[iscan+1][ichan]) * 3 / 4;
          countc_bar += (cc[iscan-2][ichan] + cc[iscan+2][ichan]) * 2 / 4;
          countc_bar += (cc[iscan-3][ichan] + cc[iscan+3][ichan]) / 4;
          countc_bar = countc_bar / 4;
          countw_bar = cw[iscan][ichan];
          countw_bar += (cw[iscan-1][ichan] + cw[iscan+1][ichan]) * 3 / 4;
          countw_bar += (cw[iscan-2][ichan] + cw[iscan+2][ichan]) * 2 / 4;
          countw_bar += (cw[iscan-3][ichan] + cw[iscan+3][ichan]) / 4;
          countw_bar = countw_bar / 4;
      }
      else if (scanneed[iscan] == 5)
      {
          countc_bar = cc[iscan][ichan];
          countc_bar += (cc[iscan-1][ichan] + cc[iscan+1][ichan]) * 2 / 3;
          countc_bar += (cc[iscan-2][ichan] + cc[iscan+2][ichan]) * 1 / 3;
          countc_bar = countc_bar / 3;
          countw_bar = cw[iscan][ichan];
          countw_bar += (cw[iscan-1][ichan] + cw[iscan+1][ichan]) * 2 / 3;
          countw_bar += (cw[iscan-2][ichan] + cw[iscan+2][ichan]) * 1 / 3;
          countw_bar = countw_bar / 3;
      }
      else if (scanneed[iscan] == 3)
      {
          countc_bar = cc[iscan][ichan];
          countc_bar += (cc[iscan-1][ichan] + cc[iscan+1][ichan]) * 1 / 2;
          countc_bar = countc_bar / 2;
          countw_bar = cw[iscan][ichan];
          countw_bar += (cw[iscan-1][ichan] + cw[iscan+1][ichan]) * 1 / 2;
          countw_bar = countw_bar / 2;
      }
      else if (scanneed[iscan] == 1)
      {
          countc_bar = cc[iscan][ichan];
          countw_bar = cw[iscan][ichan];
      }
      else
          break;

      S[iscan][ichan] = (rad_warm[iscan][ichan] - rad_cold[ichan]) / (countw_bar - countc_bar);

      for (ispot=0; ispot< NUMSPOT_A; ispot++)
      {
          Rl[iscan][ispot][ichan] = rad_cold[ichan] + S[iscan][ichan] * (count[ichan][ispot] - countc_bar);
          Z[iscan][ispot][ichan] = S[iscan][ichan] * S[iscan][ichan] * (count[ichan][ispot] - countc_bar) * (count[ichan][ispot] - countw_bar);
          calrad[ichan][ispot] = Rl[iscan][ispot][ichan] - dr[ichan] - dyear * k[ichan] + mu[ichan] *  Z[iscan][ispot][ichan];
//          if (hblock.sat_id == 4 && ichan == NUMCHAN_A - 1)		// keep 1b value for N15 Ch 15
//             calrad[ichan][ispot] = rad_warm[iscan][ichan] + S[iscan][ichan] * (count[ichan][ispot] - countw_bar) + mu15[iscan] *  Z[iscan][ispot][ichan];
//             calrad[ichan][ispot] = Rl[iscan][ispot][ichan] + mu15[iscan] * Z[iscan][ispot][ichan];
      } 

      for (ispot=0; ispot< 2; ispot++)
      {
          Rl_temp = rad_cold[ichan] + S[iscan][ichan] * (countc[ichan][ispot] - countc_bar);
          Z_temp = S[iscan][ichan] * S[iscan][ichan] * (countc[ichan][ispot] - countc_bar) * (countc[ichan][ispot] - countw_bar);
          calradc[ichan][ispot] = Rl_temp - dr[ichan] - dyear * k[ichan] + mu[ichan] *  Z_temp;
//          if (hblock.sat_id == 4 && ichan == NUMCHAN_A - 1)		// keep 1b value for N15 Ch 15
//             calradc[ichan][ispot] = rad_warm[iscan][ichan] + S[iscan][ichan] * (countc[ichan][ispot] - countw_bar) + mu15[iscan] *  Z_temp;
//             calradc[ichan][ispot] = Rl_temp + mu15[iscan] *  Z_temp;

          Rl_temp = rad_cold[ichan] + S[iscan][ichan] * (countw[ichan][ispot] - countc_bar);
          Z_temp = S[iscan][ichan] * S[iscan][ichan] * (countw[ichan][ispot] - countc_bar) * (countw[ichan][ispot] - countw_bar);
          calradw[ichan][ispot] = Rl_temp - dr[ichan] - dyear * k[ichan] + mu[ichan] *  Z_temp;
//          if (hblock.sat_id == 4 && ichan == NUMCHAN_A - 1)		// keep 1b value for N15 Ch 15
//             calradw[ichan][ispot] = rad_warm[iscan][ichan] + S[iscan][ichan] * (countw[ichan][ispot] - countw_bar) + mu15[iscan] *  Z_temp;
//             calradw[ichan][ispot] = Rl_temp + mu15[iscan] *  Z_temp;
      } 

// deal with N15 Ch 15
      if (hblock.sat_id == 4 && ichan == NUMCHAN_A - 1)
      {
          //ichan = 3;
          clcoef[ichan][2] = scanline[iscan].pri_cal_coeffs[ichan+11][0]*1.0E-19;
          clcoef[ichan][1] = scanline[iscan].pri_cal_coeffs[ichan+11][1]*1.0E-13;
          clcoef[ichan][0] = scanline[iscan].pri_cal_coeffs[ichan+11][2]*1.0E-09;

          for(ispot=0; ispot< NUMSPOT_A; ispot++)
          {
             calrad[ichan][ispot] = clcoef[ichan][0];
             calrad[ichan][ispot] = calrad[ichan][ispot] + clcoef[ichan][1]*count[ichan][ispot];
             calrad[ichan][ispot] = calrad[ichan][ispot] + clcoef[ichan][2]*count[ichan][ispot]*count[ichan][ispot];
          }

          for(ispot = 0; ispot < 2; ispot++)
          {
             calradc[ichan][ispot] = clcoef[ichan][0];
             calradc[ichan][ispot] = calradc[ichan][ispot] + clcoef[ichan][1]*countc[ichan][ispot];
             calradc[ichan][ispot] = calradc[ichan][ispot] + clcoef[ichan][2]*countc[ichan][ispot]*countc[ichan][ispot];
          }

          for(ispot = 0; ispot < 2; ispot++)
          {
             calradw[ichan][ispot] = clcoef[ichan][0];
             calradw[ichan][ispot] = calradw[ichan][ispot] + clcoef[ichan][1]*countw[ichan][ispot];
             calradw[ichan][ispot] = calradw[ichan][ispot] + clcoef[ichan][2]*countw[ichan][ispot]*countw[ichan][ispot];
          }
       }

/*---------------------------------------------------------*
 * Perform Antenna Pattern Correction
 *---------------------------------------------------------*/
      apc(ichan, numspot);

      for (ispot=0; ispot< NUMSPOT_A; ispot++)
          rad[ispot] = calrade[ichan][ispot];

/*---------------------------------------------------------*
 * Convert radiances to brightness temperatures
 *---------------------------------------------------------*/
      rad2at(ichan);    /* calculate AT from radiance */

      for (ispot=0; ispot< NUMSPOT_A; ispot++)
          at[iscan][ispot][ichan] = rat[ispot];
      

   } /* end of ichan for loop*/
   //printf("at[0][0][0] is %f.\n", at[0][0][0]);
/*   printf("countc_bar is %f, countw_bar is %f\n", countc_bar, countw_bar);
   printf("rad_warm[%d][0] is %e, t_warm[%d][0] is %e\n", iscan, rad_warm[iscan][0], iscan, t_warm[iscan][0]);
   printf("S[%d][0] is %e.\n",iscan, S[iscan][0]); 
   printf("Rl[%d][15][0] is %e.\n", iscan, Rl[iscan][15][0]);
   printf("Z[%d][15][0] is %e.\n", iscan, Z[iscan][15][0]);
   printf("dyear[%d] is %f.\n",iscan, dyear); 
*/
}/* end of getcal.c */
