/**************************************************************************
*@***h* AMSU-A Hydrological FCDR/gnrt_fcdr.c
*
* NAME
*   gnrt_fcdr.c
*
* PURPOSE
*   Subroutine to generate products
*
* DESCRIPTION
*   Program calculates ATs
*
* Input Files 
*   None
*
* Output Files
*   None 
*
* Subroutine Called 
*   qc_at.c, getcal.c
*
* Called by
*   rama_fcdr.c
*
* AUTHOR
*   Huan Meng,  NOAA/NESDIS/STAR/SCSB
*   Wenze Yang, UMD/ESSIC/CICS
*
* COPYRIGHT
*   THIS SOFTWARE AND ITS DOCUMENTATION ARE CONSIDERED TO BE IN THE PUBLIC
*   DOMAIN AND THUS ARE AVAILABLE FOR UNRESTRICTED PUBLIC USE. THEY ARE 
*   FINISHED "AS IS." THE AUTHORS, THE UNITED STATES GOVERNMENT, ITS
*   INSTRUMENTALITIES, OFFICERS, EMPLOYEES, AND AGENTS MAKE NO WARRANTY,
*   EXPRESS OR IMPLIED, AS TO THE USEFULNESS OF THE SOFTWARE AND
*   DOCUMENTATION FOR ANY PURPOSE. THEY ASSUME NO RESPONSIBILITY (1) FOR
*   THE USE OF THE SOFTWARE AND DOCUMENTATION; OR (2) TO PROVIDE TECHNICAL 
*   SUPPORT TO USERS.
*
* HISTORY
*   6/25/2015 Initial Version prepared for NCEI
*
*@*****
**************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"

/*****************************************************************/
void getcal();
void qc_at();

/*****************************************************************/
void gnrt_fcdr(short int numscan)
{

   short int   		iscan, ichan;

   unsigned int 	qa_ind;
   unsigned char        qa_time, qa_cal, qa_eloc;
   unsigned short       qa_cal_chan;
   unsigned char        a1, a2, a;

/*---------------------------------------------------------------*
 * Check if the scanline is good
 *---------------------------------------------------------------*/
//   printf("numscan: %d\n",numscan);

   for(iscan = 0; iscan < numscan; iscan++)
   {
       qa_ind = scanline[iscan].quality_indicator_bit_field;
//       printf("Quality indictor: %ld\n",qa_ind);

       orb_mode[iscan] = scanline[iscan].orbit_node>>15;

/*---------------------------------------------------------------*
 * Toss off the scanlines that are bad and flag them as 
 * "do_not_use_scan". No geophysical products are calculated
 * if bad_at = 1. This parameter is set in qc_at.c and a value
 * of 1 signals that at least one FOV of one channel out of 
 * channel 1~15 is bad and that the entire scanline is tossed 
 * for all channels  
 *--------------------------------------------------------------*/
       if (qa_ind == 0)   /* good scanline */
       {
           getcal(iscan, numscan);

           qc_at(iscan);

       }  
       else /* qa_ind ~=0 */
       {

       qa_time = scanline[iscan].qa_time;
       qa_cal = scanline[iscan].qa_cal;
       qa_eloc = scanline[iscan].qa_eloc;

// test bit 31 of Quality Indicator Bit Field
       if (qa_ind & (1<<31) != 0)
          for (ichan = 0; ichan < NUMCHAN_A; ichan++)
            qa_prod[iscan][ichan] |= 1<<7;

// test qa_time
       if (qa_time != 0)
          for (ichan = 0; ichan < NUMCHAN_A; ichan++)
            qa_prod[iscan][ichan] |= 1<<5;

// test qa_eloc
       if (qa_eloc != 0)
          for (ichan = 0; ichan < NUMCHAN_A; ichan++)
            qa_prod[iscan][ichan] |= 1<<4;

// test qa_cal_chan
       for (ichan = 0; ichan < NUMCHAN_A; ichan++)
       {
          qa_cal_chan = scanline[iscan].qa_cal_chan[ichan];
          if (qa_cal_chan & (7<<3) != 0)
             qa_prod[iscan][ichan] |= 1<<6;
          if (qa_cal_chan & (3<<6) != 0)
             qa_prod[iscan][ichan] |= 1<<2;
       } /* end of ichan loop */


       } /* end of qa_ind check */

    }   /* end of iscan loop */

}  /* end of gnrt_prod.c  */
