/***************************************************************************
 *  Program Name      : gnrt_fname.c
 *  Type              : Subroutine
 *  Function          : Program generates AMSU-A swath file name 
 *  Input Files       : None 
 *  Output Files      : SWATH_MOA_AMSUA_Syyddd_sttttt_Eyyddd_ettttt.nc
 *			(yy: year, ddd: julian day, sttttt: starting time,
 *			 ettttt: ending time)
 *  Subroutine Called : None 
 *  Called by         : rama_wfcdr.c
 *
 *  Modification History:
 *      Date       Version         Description of Changes
 *     ------     ---------       ------------------------
 *   8/30/2000      v2.0
 *************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"
#include <string.h>

/*****************************************************************/
//char* gnrt_fname(char* nc_filename)
void gnrt_fname(char* nc_filename, char direct[], char fname_1b[], short int numscan)
{

   char     temp[80];

   char     nc[] = {".nc"};
   char     *token;
   char     *yd;
   char     *search = ".";
   int      i;
   char     buffer[80], sstr[80], estr[80];
   int      hh, mm, ss;

   switch (hblock.sat_id)
   {
      case 4: 
         sprintf(sid, "N15"); 
         snum = 0;
         sprintf(platform, "NOAA-15 > National Oceanic and Atmospheric Administration - 15");
         break;
      case 2: 
         sprintf(sid, "N16"); 
         snum = 1;
         sprintf(platform, "NOAA-16 > National Oceanic and Atmospheric Administration - 16");
         break;
      case 6: 
         sprintf(sid, "N17"); 
         snum = 2;
         sprintf(platform, "NOAA-17 > National Oceanic and Atmospheric Administration - 17");
         break;
      case 7: 
         sprintf(sid, "N18"); 
         snum = 3;
         sprintf(platform, "NOAA-18 > National Oceanic and Atmospheric Administration - 18");
         break;
      case 8: 
         sprintf(sid, "N19"); 
         snum = 5;
         sprintf(platform, "NOAA-19 > National Oceanic and Atmospheric Administration - 19");
         break;
      case 12: 
         sprintf(sid, "MOA"); 
         snum = 4;
         sprintf(platform, "MetOp-A > Meteorological Operational satellite programme - A");
         break;
      default: printf("wrong satellite id\n"); break;
   }

   yd = strtok(fname_1b, search);
   for (i=0; i<3;i++) 
      yd = strtok(NULL, search); 
   for (i=0; i<3;i++) 
      token = strtok(NULL, search); 

   sscanf(str_scantime[0],"%11c%2d:%2d:%2d",buffer,&hh,&mm,&ss);
   sprintf(sstr,"S%.2d%.2d%.2d",hh,mm,ss);
   sscanf(str_scantime[numscan-1],"%11c%2d:%2d:%2d",buffer,&hh,&mm,&ss);
   sprintf(estr,"E%.2d%.2d%.2d",hh,mm,ss);

   sprintf(temp,"CICS_V00R01_AMSUA_FCDR_%s_%s_%s_%s_%s", sid, yd, sstr, estr, token);
//   sprintf(temp,"CICS_V01R00_AMSUA_FCDR_%s_%s_S%.6ld_E%.6ld_%s", sid, yd, stime, etime, token);
   strcpy(nc_filename,direct);

   strcpy(nc_fname,temp);
   strcat(nc_fname,nc);

   strcat(nc_filename, nc_fname);

   printf("gnrt_fname/filename: %s\n",nc_filename);
   printf("gnrt_fname/filename: %s\n",nc_fname);
   
} /* end of gnrt_fname.c */                             
