/**************************************************************************
 *  Program Name      : gnrt_stime.c
 *  Type              : Subroutine
 *  Function          : Program sets up the scan time for geo table 
 *  Input Files       : None
 *  Output Files      : None
 *  Subroutine Called : leapyear.c, jd2md.c
 *  Called by         : rama_wfcdr.c
 *
 *  Modification History:
 *      Date       Version         Description of Changes
 *     ------     ---------       ------------------------
 *   8/30/2000      v2.0
 *************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"

/*****************************************************************/
long int leapyear();
void     jd2md();

/*************************************************************/
void gnrt_stime(short int numscan)
{

   unsigned short 	jday, sday, eday;
   unsigned long        itotsec, stotsec;
   unsigned short       stime, syear; 
   long int		lyflag, hour, min, sec;
   long int		totsec, is_day_cross=0;
   short int   		iscan, ifov, mon, domth;
  
   short int  		lzyear;
   short int  		lzdoy;
   char       		lzmonth, lzdom;
   char       		lzhour, lzminute, lzsecond;
   time_t		rawtime;
   struct tm* 		tis;
   char       		buffer[80];

   time_t 		TAI93;
   struct tm*           T93;
	
/*-----------------------------------------------------*
 * Extract hour, minute and second, and set up the 
 * scanline julian day by looking up the day cross flag
 *-----------------------------------------------------*/
   sday  = hblock.start_day_of_year;
   eday  = hblock.end_day_of_year;
   if(sday != eday) is_day_cross = 1;
   jday = sday;

   stime = hblock.start_year;

   syear = stime;

   lyflag = leapyear(syear);

   stotsec = hblock.start_milliseconds_of_day;

   time(&rawtime);
   T93 = gmtime(&rawtime);
   T93->tm_sec=0;               // Seconds.     [0-60] (1 leap second) 
   T93->tm_min=0;	        // Minutes.     [0-59] 
   T93->tm_hour=0;	        // Hours.       [0-23] 
   T93->tm_mday=1;	        // Day.         [1-31] 
   T93->tm_mon=0;	        // Month.       [0-11] 
   T93->tm_year=98;	        // Year - 1900.  
   T93->tm_wday=4;	        // Day of week. [0-6] 
   T93->tm_yday=0;	        // Days in year.[0-365] 
   T93->tm_isdst=0;		// DST.         [-1/0/1]

   TAI93 = mktime(T93);

   for (iscan = 0; iscan < numscan; iscan++)
   {
      itotsec = scanline[iscan].time_of_day_of_scan;
      totsec = itotsec / 1000;
      hour  = totsec / 3600;
      min = (totsec - hour*3600) / 60;
      sec = totsec - hour * 3600 - min * 60;

/*-----------------------------------------------------*
 * Decide the day of year/month and month of year 
 *-----------------------------------------------------*/
      if(is_day_cross == 1)
         if (itotsec < stotsec)
         {
            jday = sday + 1;

/* Change of the year  */
            if( (lyflag && jday == 367) || (!lyflag && jday == 366) )
            {
                jday = 1;
                syear = syear + 1;

/* Change of the century  */
                if(syear == 100)
                   syear = 0;
            }
         }

      jd2md(jday, &domth, &mon);   

      lzyear = syear;
      lzmonth = mon;
      lzdom   = domth;  
      lzhour  = hour;
      lzminute = min;
      lzsecond = sec;
      lzdoy    = jday; 

      geo_scantime[iscan].year   = lzyear;
      geo_scantime[iscan].month  = lzmonth;
      geo_scantime[iscan].dom    = lzdom;
      geo_scantime[iscan].hour   = lzhour;
      geo_scantime[iscan].minute = lzminute;
      geo_scantime[iscan].second = lzsecond;
      geo_scantime[iscan].doy    = lzdoy;
      
//      for(ifov = 0; ifov < NUMSPOT_A; ifov++)
//        b_time[iscan][ifov] = lzhour * 3600 + lzminute * 60 + lzsecond;

      time(&rawtime);
      tis = gmtime(&rawtime);
      tis->tm_year = geo_scantime[iscan].year - 1900;
      tis->tm_mon = geo_scantime[iscan].month - 1;
      tis->tm_mday = geo_scantime[iscan].dom;
      tis->tm_hour = geo_scantime[iscan].hour;
      tis->tm_min = geo_scantime[iscan].minute;
      tis->tm_sec = geo_scantime[iscan].second;
      strftime(buffer,80,"%Y-%m-%dT%H:%M:%S",tis);

      if ( mon >= 11 || mon < 4 ) 
	 tis->tm_isdst=0;	      	/* DST.	      [-1/0/1]*/
      else
	 tis->tm_isdst=1;    
	
      //str_scantime[iscan] = malloc(19);
      strncpy(str_scantime[iscan],buffer,19);
      time_tai93[iscan] = difftime( mktime(tis), TAI93 );
//      time_tai93[iscan] = difftime( mktime(tis), TAI93 ) + 11;
   }

} /* end of gnrt_stime.c */
