#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <math.h>
#define NR_END 1
#define FREE_ARG char*

void nrerror();
float *vector();
void free_vector();
void polint(float xa[], float ya[], int n, float x, float *y, float *dy);

void polcof(float xa[], float ya[], int n, float cof[])
{
	int k,j,i;
	float xmin,dy,*x,*y;
	x=vector(0,n);
	y=vector(0,n);
	for (j=0;j<=n;j++) {
	    x[j]=xa[j];
	    y[j]=ya[j];
	}
	for (j=0;j<=n;j++) {
	    polint(x-1,y-1,n+1-j,0.0,&cof[j],&dy);
	    xmin=1.0e38;
	    k = -1;
	    for (i=0;i<=n-j;i++) { //Find the remaining xi of smallest
		if (fabs(x[i]) < xmin) { //absolute value,
		    xmin=fabs(x[i]);
		    k=i;
		}
		if (x[i]) y[i]=(y[i]-cof[j])/x[i]; //(meanwhile reducing all the terms)
	    }
	    for (i=k+1;i<=n-j;i++) { //and eliminate it.
		y[i-1]=y[i];
		x[i-1]=x[i];
	    }
	}
	free_vector(y,0,n);
	free_vector(x,0,n);
}

void polint(float xa[], float ya[], int n, float x, float *y, float *dy)
{
	int i,m,ns=1;
	float den,dif,dift,ho,hp,w;
	float *c,*d;

	dif=fabs(x-xa[1]);
	c=vector(1,n);
	d=vector(1,n);
	for (i=1;i<=n;i++) { //Here we find the index ns of the closest table entry,
	    if ( (dift=fabs(x-xa[i])) < dif) {
		ns=i;
		dif=dift;
	    }
	    c[i]=ya[i]; //and initialize the tableau of c’s and d’s.
	    d[i]=ya[i];
	}
	*y=ya[ns--]; //This is the initial approximation to y.
	for (m=1;m<n;m++) { //For each column of the tableau,
	    for (i=1;i<=n-m;i++) { //we loop over the current c’s and d’s and update
		ho=xa[i]-x; //them.
		hp=xa[i+m]-x;
		w=c[i+1]-d[i];
		if ( (den=ho-hp) == 0.0) nrerror("Error in routine polint");
		//This error can occur only if two input xa’s are (to within roundoff) identical.
		den=w/den;
		d[i]=hp*den; //Here the c’s and d’s are updated.
		c[i]=ho*den;
	    }
	    *y += (*dy=(2*ns < (n-m) ? c[ns+1] : d[ns--]));
//After each column in the tableau is completed, we decide which correction, c or d,
//we want to add to our accumulating value of y, i.e., which path to take through the
//tableau—forking up or down. We do this in such a way as to take the most “straight
//line” route through the tableau to its apex, updating ns accordingly to keep track of
//where we are. This route keeps the partial approximations centered (insofar as possible)
//on the target x. The last dy added is thus the error indication.
	}
	free_vector(d,1,n);
	free_vector(c,1,n);
}

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
fprintf(stderr,"Numerical Recipes run-time error...\n");
fprintf(stderr,"%s\n",error_text);
fprintf(stderr,"...now exiting to system...\n");
exit(1);
}
float *vector(long nl, long nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
float *v;
v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
if (!v) nrerror("allocation failure in vector()");
return v-nl+NR_END;
}
void free_vector(float *v, long nl, long nh)
/* free a float vector allocated with vector() */
{
free((FREE_ARG) (v+nl-NR_END));
}
