/***************************************************************************
 *  Program Name      : rama_hd.c
 *  Type              : Subroutine
 *  Function          : Program reads the header record of AMSU-A 1B* file 
 *  Input Files       : AMAX.NL.Dyyddd.Ssttt.Eettt
 *			(yy: year, ddd: julian day, sttt: starting time,
 *			 ettt: ending time) 
 *  Output Files      : None
 *  Subroutine Called : polcof.c, t2rad.c 
 *  Called by         : rama_wfcdr.c
 *
 *  Modification History:
 *      Date       Version         Description of Changes
 *     ------     ---------       ------------------------
 *   08/30/2000      v2.0
 *   05/09/2006                   Change to reading from 1b instead of 1b*
 *************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"
void polcof(); 
void t2rad(int chan, float t, float *radt);

/**************************************************************/
void rama_hd(FILE *ama1b)
{
   int n=2,ich,i;
   int p2ch[NUMCHAN_A] = {3,3,2,1}; 
   int a2ch[NUMCHAN_A] = {1,2,3,15}; 
   float q1[3], ss[3], cof[3];
/*---------------------------------------------------*
 * Read the header block
 *---------------------------------------------------*/
   fread(&hblock, L_HBLOCK, 1, ama1b);
    
/*---------------------------------------------------*
 * Put central wave numbers in an array 
 *---------------------------------------------------*/
   t_r_central_wave_number[0] = hblock.t_r_central_wave_number_1*1.0E-6;
   t_r_central_wave_number[1] = hblock.t_r_central_wave_number_2*1.0E-6;
   t_r_central_wave_number[2] = hblock.t_r_central_wave_number_3*1.0E-6;
   t_r_central_wave_number[3] = hblock.t_r_central_wave_number_15*1.0E-6;

// Calculate DeltaTw coef, Tc, dcoef and scoef
   for (ich = 0; ich < NUMCHAN_A; ich ++)
   {
     for (i = 0; i < 3; i ++)
     {
       q1[i] = hblock.rfs[p2ch[ich]-1][i] * 1.0e-2;
       ss[i] = hblock.corr[a2ch[ich]-1][i] * 1.0e-3;
     }
     polcof(q1, ss, n, cof);
     for (i = 0; i < 3; i ++)
       rfs_corr[ich][i] = cof[i];

     t_cold[ich] = 2.73 + hblock.corr[a2ch[ich]-1][3] * 1.0e-3;
     
     t2rad(ich, t_cold[ich], &rad_cold[ich]);
   }

// Calculate mu_coef for N15, Ch 15
/*   if (hblock.sat_id == 4)
   {
     for (i = 0; i < 3; i ++)
     {
       q1[i] = hblock.rfs[1-1][i] * 1.0e-2;
       ss[i] = hblock.mu[15-1][i] * 1.0e-6;
     }
     polcof(q1, ss, n, mu_coef);
   }
*/
// Calculate dcoef and scoef
   for (ich = 0; ich < 10; ich ++)
   {
     dcoef_a1[ich][0] = hblock.warm_digital_coeffs_a1[ich][0] * 1.0e-4;
     dcoef_a1[ich][1] = hblock.warm_digital_coeffs_a1[ich][1] * 1.0e-9;
     dcoef_a1[ich][2] = hblock.warm_digital_coeffs_a1[ich][2] * 1.0e-16;
     dcoef_a1[ich][3] = hblock.warm_digital_coeffs_a1[ich][3] * 1.0e-20;
   }

   for (ich = 0; ich < 7; ich ++)
   {
     dcoef_a2[ich][0] = hblock.warm_digital_coeffs_a2[ich][0] * 1.0e-4;
     dcoef_a2[ich][1] = hblock.warm_digital_coeffs_a2[ich][1] * 1.0e-9;
     dcoef_a2[ich][2] = hblock.warm_digital_coeffs_a2[ich][2] * 1.0e-16;
     dcoef_a2[ich][3] = hblock.warm_digital_coeffs_a2[ich][3] * 1.0e-20;
   }

   for (ich = 0; ich < 2; ich ++)
   {
     scoef_a1[ich][0] = hblock.rfs_coeffs_a1[ich][0] * 1.0e-4;
     scoef_a1[ich][1] = hblock.rfs_coeffs_a1[ich][1] * 1.0e-9;
     scoef_a1[ich][2] = hblock.rfs_coeffs_a1[ich][2] * 1.0e-16;
     scoef_a1[ich][3] = hblock.rfs_coeffs_a1[ich][3] * 1.0e-20;
   }

     scoef_a2[0] = hblock.rfs_coeffs_a2[0] * 1.0e-4;
     scoef_a2[1] = hblock.rfs_coeffs_a2[1] * 1.0e-9;
     scoef_a2[2] = hblock.rfs_coeffs_a2[2] * 1.0e-16;
     scoef_a2[3] = hblock.rfs_coeffs_a2[3] * 1.0e-20;

/*---------------------------------------------------*
 * Print out header block information
 *---------------------------------------------------*/
/*   printf("\n-----------------------------------------------------\n");
   printf(" Start Year             :  %d\n", hblock.start_year);
   printf(" Start Day of Year      :  %d\n", hblock.start_day_of_year);
   printf(" End Year               :  %d\n", hblock.end_year);
   printf(" End Day of Year        :  %d\n", hblock.end_day_of_year);
   printf(" Central Wave Number 1  :  %ld\n", hblock.t_r_central_wave_number_1);
   printf(" Central Wave Number 2  :  %ld\n", hblock.t_r_central_wave_number_2);
   printf(" Central Wave Number 3  :  %ld\n", hblock.t_r_central_wave_number_3);
   printf(" Central Wave Number 4  :  %ld\n", hblock.t_r_central_wave_number_4);
   printf(" Central Wave Number 5  :  %ld\n", hblock.t_r_central_wave_number_5);
   printf(" Central Wave Number 6  :  %ld\n", hblock.t_r_central_wave_number_6);
   printf(" Central Wave Number 7  :  %ld\n", hblock.t_r_central_wave_number_7);
   printf(" Central Wave Number 8  :  %ld\n", hblock.t_r_central_wave_number_8);
   printf(" Central Wave Number 9  :  %ld\n", hblock.t_r_central_wave_number_9);
   printf(" Central Wave Number 10 :  %ld\n", hblock.t_r_central_wave_number_10);
   printf(" Central Wave Number 11 :  %ld\n", hblock.t_r_central_wave_number_11);
   printf(" Central Wave Number 12 :  %ld\n", hblock.t_r_central_wave_number_12);
   printf(" Central Wave Number 13 :  %ld\n", hblock.t_r_central_wave_number_13);
   printf(" Central Wave Number 14 :  %ld\n", hblock.t_r_central_wave_number_14);
   printf(" Central Wave Number 15 :  %ld\n", hblock.t_r_central_wave_number_15);

   printf("-----------------------------------------------------\n");

   printf(" rfs_coeffs_a1: %d\n", hblock.rfs_coeffs_a1[0][0]);
   printf(" warm_digital_coeffs_a1: %d\n", hblock.warm_digital_coeffs_a1[0][0]);
   printf(" rfs_coeffs_a2: %d\n", hblock.rfs_coeffs_a2[0]);
   printf(" warm_digital_coeffs_a2: %d\n", hblock.warm_digital_coeffs_a2[0][0]);
   printf(" size of head: %d\n", sizeof(hblock));
   printf(" size of data: %d\n", sizeof(scanline));
   i = 14;
   printf(" corr[0][0~3]: %d %d %d %d\n", hblock.corr[i][0], hblock.corr[i][1], hblock.corr[i][2], hblock.corr[i][3]);
   i = 0;
   printf(" rfs_corr[0][0~3]: %f %f %f \n", rfs_corr[i][0], rfs_corr[i][1], rfs_corr[i][2]);
   printf(" t_cold: %f %f %f %f\n", t_cold[0], t_cold[1], t_cold[2], t_cold[3]); 
   printf(" rad_cold: %f %f %f %f\n", rad_cold[0], rad_cold[1], rad_cold[2], rad_cold[3]); 
   printf(" dcoef_a2[0][0~4]: %e %e %e %e\n", dcoef_a2[ich][0], dcoef_a2[ich][1], dcoef_a2[ich][2], dcoef_a2[ich][3]);
   printf(" scoef_a2[0~4]: %e %e %e %e\n", scoef_a2[0], scoef_a2[1], scoef_a2[2], scoef_a2[3]);
//*/
//   printf(" mu[0~2]: %f %f %f \n", ss[0], ss[1], ss[2]);
} /* end of rama_hd.c */
