/***************************************************************************
 *  Program Name      : rama_scn.c
 *  Type              : Subroutine
 *  Function          : Program reads scanline data from AMSU-A 1B* file
 *  Input Files       : AMAX.NL.Dyyddd.Ssttt.Eettt
 *                      (yy: year, ddd: julian day, sttt: starting time,
 *                       ettt: ending time) 
 *  Output Files      : None
 *  Subroutine Called : t2rad2.c 
 *  Called by         : rama_wfcdr.c
 *
 *  Modification History:
 *      Date       Version         Description of Changes
 *     ------     ---------       ------------------------
 *   8/30/2000      v2.0
 *************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"
#include <math.h>
void t2rad2();

/******************************************************************/
void rama_scn(FILE *ama1b, short int numscan)
{
   short int       iscan,i,numt,ich, ichan;
   int 		   p2ch[NUMCHAN_A] = {3,3,2,1}; 
   int 		   a2ch[NUMCHAN_A] = {1,2,3,15}; 
   unsigned short  cwd;	
   float	   dt = 0.2, sumt;

/* -------------------------------------------------*
 * Initilize scanneed vector first
 * Then modify it
 *---------------------------------------------------*/

    for(iscan = 0; iscan < numscan; iscan++) {    
       if (iscan >= 3 && iscan <= numscan - 4)
           scanneed[iscan] = 7;
       else if (iscan == 2 || iscan == numscan - 3)
           scanneed[iscan] = 5;
       else if (iscan == 1 || iscan == numscan - 2)
           scanneed[iscan] = 3;
       else
           scanneed[iscan] = 1;
    }

/*---------------------------------------------------*
 * Read AMSU-A 1B* scanline data records
 * And calculate twd, t_rfs, twd_weight, dtw, tw
 *---------------------------------------------------*/
   for(iscan = 0; iscan < numscan; iscan++)
   {
     fread(&scanline[iscan], L_SCANLINE, 1, ama1b);
 
     for (i = 0; i < 10; i ++)
     {
       cwd = scanline[iscan].countwd_a1[i];
       twd_a1[iscan][i] = dcoef_a1[i][0];
       twd_a1[iscan][i] += dcoef_a1[i][1] * cwd; 
       twd_a1[iscan][i] += dcoef_a1[i][2] * cwd * cwd; 
       twd_a1[iscan][i] += dcoef_a1[i][3] * cwd * cwd * cwd; 
     }
 
     for (i = 0; i < 7; i ++)
     {
       cwd = scanline[iscan].countwd_a2[i];
       twd_a2[iscan][i] = dcoef_a2[i][0];
       twd_a2[iscan][i] += dcoef_a2[i][1] * cwd; 
       twd_a2[iscan][i] += dcoef_a2[i][2] * cwd * cwd; 
       twd_a2[iscan][i] += dcoef_a2[i][3] * cwd * cwd * cwd; 
     }

     for (i = 0; i < 2; i ++)
     {
       cwd = scanline[iscan].count_rfs_a1[i];
       t_rfs_a1[iscan][i] = scoef_a1[i][0];
       t_rfs_a1[iscan][i] += scoef_a1[i][1] * cwd; 
       t_rfs_a1[iscan][i] += scoef_a1[i][2] * cwd * cwd; 
       t_rfs_a1[iscan][i] += scoef_a1[i][3] * cwd * cwd * cwd; 
     }

       cwd = scanline[iscan].count_rfs_a2;
       t_rfs_a2[iscan] = scoef_a2[0];
       t_rfs_a2[iscan] += scoef_a2[1] * cwd; 
       t_rfs_a2[iscan] += scoef_a2[2] * cwd * cwd; 
       t_rfs_a2[iscan] += scoef_a2[3] * cwd * cwd * cwd; 

     if (iscan == 0)
     {
       numt = 5;

       sumt = 0;
       for(i=0;i<5;i++) sumt += twd_a1[iscan][i];
       twd_weight[iscan][0] = sumt / numt;
       sumt = 0;
       for(i=0;i<5;i++) sumt += twd_a1[iscan][i+5];
       twd_weight[iscan][1] = sumt / numt;

       numt = 7;
       sumt = 0;
       for(i=0;i<7;i++) sumt += twd_a2[iscan][i];
       twd_weight[iscan][2] = sumt / numt;
     }
     else
     {
       numt = 0;
       sumt = 0;
       for (i=0;i<5;i++)
       {
         if (fabs(twd_a1[iscan][i] - twd_a1[iscan-1][i]) < dt) 
         {
            numt += 1;
            sumt += twd_a1[iscan][i];
         }
       }
       if (numt > 0) twd_weight[iscan][0] = sumt / numt;

       numt = 0;
       sumt = 0;
       for (i=0;i<5;i++)
       {
         if (fabs(twd_a1[iscan][i+5] - twd_a1[iscan-1][i+5]) < dt) 
         {
            numt += 1;
            sumt += twd_a1[iscan][i+5];
         }
       }
       if (numt > 0) twd_weight[iscan][1] = sumt / numt;

       numt = 0;
       sumt = 0;
       for (i=0;i<7;i++)
       {
         if (fabs(twd_a2[iscan][i] - twd_a2[iscan-1][i]) < dt) 
         {
            numt += 1;
            sumt += twd_a2[iscan][i];
         }
       }
       if (numt > 0) twd_weight[iscan][2] = sumt / numt;
     }

     for (ich = 0; ich < 2; ich ++)
     {
       dtw[iscan][ich] = rfs_corr[ich][0];
       dtw[iscan][ich] += rfs_corr[ich][1] * t_rfs_a2[iscan];
       dtw[iscan][ich] += rfs_corr[ich][2] * t_rfs_a2[iscan] * t_rfs_a2[iscan];
     }

     ich = 2;
       dtw[iscan][ich] = rfs_corr[ich][0];
       dtw[iscan][ich] += rfs_corr[ich][1] * t_rfs_a1[iscan][2];
       dtw[iscan][ich] += rfs_corr[ich][2] * t_rfs_a1[iscan][2] * t_rfs_a1[iscan][2];

     ich = 3;
       dtw[iscan][ich] = rfs_corr[ich][0];
       dtw[iscan][ich] += rfs_corr[ich][1] * t_rfs_a1[iscan][1];
       dtw[iscan][ich] += rfs_corr[ich][2] * t_rfs_a1[iscan][1] * t_rfs_a1[iscan][1];

     for (ich = 0; ich < NUMCHAN_A; ich ++)
       t_warm[iscan][ich] = twd_weight[iscan][p2ch[ich]-1] + dtw[iscan][ich];

// Calculate mu_coef for N15, Ch 15
/*     if (hblock.sat_id == 4)
     {
       mu15[iscan] = mu_coef[0];
       mu15[iscan] += mu_coef[1] * t_rfs_a1[iscan][1];
       mu15[iscan] += mu_coef[2] * t_rfs_a1[iscan][1] * t_rfs_a1[iscan][1];
     }
*/
     for(ichan = 0; ichan < 2; ichan++)
     {
       countc[ichan][0] = scanline[iscan].cold_cal_telemetry_a2[ichan+2] - scanline[iscan].countc_corr[a2ch[ichan]-1];
       countc[ichan][1] = scanline[iscan].cold_cal_telemetry_a2[ichan+4] - scanline[iscan].countc_corr[a2ch[ichan]-1];
       countw[ichan][0] = scanline[iscan].warm_cal_telemetry_a2[ichan+2];
       countw[ichan][1] = scanline[iscan].warm_cal_telemetry_a2[ichan+4];
     }

     ichan = 2;
     countc[ichan][0] = scanline[iscan].cold_cal_telemetry_a1[ichan+2] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countc[ichan][1] = scanline[iscan].cold_cal_telemetry_a1[ichan+15] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countw[ichan][0] = scanline[iscan].warm_cal_telemetry_a1[ichan+2];
     countw[ichan][1] = scanline[iscan].warm_cal_telemetry_a1[ichan+15];

     ichan = 3;
     countc[ichan][0] = scanline[iscan].cold_cal_telemetry_a1[ichan+2+11] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countc[ichan][1] = scanline[iscan].cold_cal_telemetry_a1[ichan+15+11] - scanline[iscan].countc_corr[a2ch[ichan]-1];
     countw[ichan][0] = scanline[iscan].warm_cal_telemetry_a1[ichan+2+11];
     countw[ichan][1] = scanline[iscan].warm_cal_telemetry_a1[ichan+15+11];

     for (ichan = 0; ichan < NUMCHAN_A; ichan ++)
     {
       cc[iscan][ichan] = (float)(countc[ichan][0] + countc[ichan][1]) / 2;
       cw[iscan][ichan] = (float)(countw[ichan][0] + countw[ichan][1]) / 2;
     }        

// note t_warm should arround 285 K
     if (cc[iscan][0] <= 0 || cw[iscan][0] <= 0 || t_warm[iscan][0] <= 100)
     {
         scanneed[iscan] = 0;

         if (iscan >= 1)
             scanneed[iscan-1] = scanneed[iscan-1] > 1 ? 1 : scanneed[iscan-1];
         if (iscan >= 2)
             scanneed[iscan-2] = scanneed[iscan-2] > 3 ? 3 : scanneed[iscan-2];
         if (iscan >= 3)
             scanneed[iscan-3] = scanneed[iscan-3] > 5 ? 5 : scanneed[iscan-3];
         if (iscan <= numscan - 4)
             scanneed[iscan+3] = scanneed[iscan+3] > 5 ? 5 : scanneed[iscan+3];
         if (iscan <= numscan - 3)
             scanneed[iscan+2] = scanneed[iscan+2] > 3 ? 3 : scanneed[iscan+2];
         if (iscan <= numscan - 2)
             scanneed[iscan+1] = scanneed[iscan+1] > 1 ? 1 : scanneed[iscan+1];

/*
         if (iscan >= 3 && iscan <= numscan - 4) {
             scanneed[iscan+1] = scanneed[iscan+1] > 1 ? 1 : scanneed[iscan+1];
             scanneed[iscan-1] = scanneed[iscan-1] > 1 ? 1 : scanneed[iscan-1];
             scanneed[iscan+2] = scanneed[iscan+2] > 3 ? 3 : scanneed[iscan+2];
             scanneed[iscan-2] = scanneed[iscan-2] > 3 ? 3 : scanneed[iscan-2];
             scanneed[iscan+3] = scanneed[iscan+3] > 5 ? 5 : scanneed[iscan+3];
             scanneed[iscan-3] = scanneed[iscan-3] > 5 ? 5 : scanneed[iscan-3];
         }
         else if (iscan == 2 || iscan == numscan - 3) {
             scanneed[iscan+1] = scanneed[iscan+1] > 1 ? 1 : scanneed[iscan+1];
             scanneed[iscan-1] = scanneed[iscan-1] > 1 ? 1 : scanneed[iscan-1];
             scanneed[iscan+2] = scanneed[iscan+2] > 3 ? 3 : scanneed[iscan+2];
             scanneed[iscan-2] = scanneed[iscan-2] > 3 ? 3 : scanneed[iscan-2];
         }
         else if (iscan == 1 || iscan == numscan - 2) {
             scanneed[iscan+1] = scanneed[iscan+1] > 1 ? 1 : scanneed[iscan+1];
             scanneed[iscan-1] = scanneed[iscan-1] > 1 ? 1 : scanneed[iscan-1];
         }
         else if (iscan == 0)
             scanneed[iscan+1] = scanneed[iscan+1] > 1 ? 1 : scanneed[iscan+1];
         else
             scanneed[iscan-1] = scanneed[iscan-1] > 1 ? 1 : scanneed[iscan-1];          
*/
     }
/*---------------------------------------------------*
 * Print out some scanline information
 *---------------------------------------------------*/
/*
     printf(" scan num    :  %d\n",scanline[iscan].scan_line_number);
     printf(" year-scan   :  %d\n",scanline[iscan].year_of_scan);
     printf(" d/y of scan :  %d\n",scanline[iscan].day_of_year_of_scan);
     printf(" t/d of scan :  %d\n",scanline[iscan].time_of_day_of_scan);
     printf("---------------------------------\n");
*/
//printf("%d: countc is %d, %d, countw is %d, %d\n", iscan, countc[0][0], countc[0][1], countw[0][0], countw[0][1]);
   }
   t2rad2(numscan, t_warm, rad_warm);
//   for (iscan = 0; iscan < numscan; iscan ++) 
//      printf("iscan is %d, scanneed is %d, cc is %f, cw is %f\n", iscan, scanneed[iscan], cc[iscan][0], cw[iscan][0]);

/*   i = 0;
   printf(" twd_a1[%d][0] : %f \n", i, twd_a1[i][0]);
   printf(" twd_a2[%d][0] : %f \n", i, twd_a2[i][0]);
   printf(" t_rfs_a1[%d][0] : %f \n", i, t_rfs_a1[i][0]);
   printf(" t_rfs_a2[%d] : %f \n", i, t_rfs_a2[i]);
   printf(" twd_weight[%d~%d][2]: %f %f \n", i, i+1, twd_weight[i][2], twd_weight[i+1][2]);
   printf(" dtw[%d~%d][0]: %f %f \n", i, i+1, dtw[i][0], dtw[i+1][0]);
   printf(" t_warm[%d~%d][0]: %f %f \n", i, i+1, t_warm[i][0], t_warm[i+1][0]);
   printf(" rad_warm[%d~%d][0]: %f %f \n", i, i+1, rad_warm[i][0], rad_warm[i+1][0]);
*/
} /* end of rama_scn.c */
