/***************************************************************************
 *  Program Name      : read_geodata.c
 *  Type              : Subroutine
 *  Function          : Program reads geo data from GEO 1B* file
 *  Input Files       : ATT1.CHc.NSS.AMAX.NL.Dyyddd.Ssttt.Eettt
 *                      (c: channel, yy: year, ddd: julian day, sttt: starting time,
 *                       ettt: ending time) 
 *  Output Files      : None
 *  Subroutine Called : None 
 *  Called by         : rama_wfcdr.c
 *
 *  Modification History:
 *      Date       Version         Description of Changes
 *     ------     ---------       ------------------------
 *   7/19/2011      v1.0
 *************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"

/******************************************************************/
FILE *openfs();
float ftrunc(float var, float scale);

/******************************************************************/
int read_geodata(char *geo_direct, char *fname, short int numscan)
{
   short int       iscan, ifov;
   char            a11_fname[120], a12_fname[120], a2_fname[120];
   FILE        	   *fp_a11_geo, *fp_a12_geo, *fp_a2_geo;

/*---------------------------------------------------*
 * Decide geolocation file names
 *---------------------------------------------------*/
   strcpy(a11_fname, geo_direct);
   strcat(a11_fname, "ATT1.CH15.");
   strcat(a11_fname, fname);

   strcpy(a12_fname, geo_direct);
   strcat(a12_fname, "ATT1.CH3.");
   strcat(a12_fname, fname);

   strcpy(a2_fname, geo_direct);
   strcat(a2_fname, "ATT1.CH1.");
   strcat(a2_fname, fname);

/*---------------------------------------------------*
 * Open geolocation files
 *---------------------------------------------------*/
   fp_a11_geo = openfs(a11_fname);
   fp_a12_geo = openfs(a12_fname);
   fp_a2_geo = openfs(a2_fname);

   if (fp_a11_geo == NULL || fp_a12_geo == NULL || fp_a2_geo == NULL) 
     return(1);

/*---------------------------------------------------*
 * Read geolocation data records
 *---------------------------------------------------*/

   fread(&geohead, L_GEOHEAD, 1, fp_a11_geo);
   fread(&geohead, L_GEOHEAD, 1, fp_a12_geo);
   fread(&geohead, L_GEOHEAD, 1, fp_a2_geo);

   for(iscan = 0; iscan < numscan; iscan++)
      for(ifov = 0; ifov < NUMSPOT_A; ifov++)
      {
        fread(&geoline, L_GEOLINE, 1, fp_a11_geo);
        lat_a1_1[iscan][ifov] = geoline.lat*1.0E-5; 
        lon_a1_1[iscan][ifov] = geoline.lon*1.0E-5; 
        lat_a1_1[iscan][ifov] = ftrunc(lat_a1_1[iscan][ifov], 1000.0); 
        lon_a1_1[iscan][ifov] = ftrunc(lon_a1_1[iscan][ifov], 1000.0); 
        if(ifov < NUMSPOT_A/2)
        { 
          lza_a1_1[iscan][ifov] = -geoline.lza*1.0E-5; 
          ssa_a1_1[iscan][ifov] = -geoline.ssa*1.0E-5;
        } 
        else
        {
          lza_a1_1[iscan][ifov] = geoline.lza*1.0E-5; 
          ssa_a1_1[iscan][ifov] = geoline.ssa*1.0E-5; 
        }
        lza_a1_1[iscan][ifov] = ftrunc(lza_a1_1[iscan][ifov], 100.0); 
        ssa_a1_1[iscan][ifov] = ftrunc(ssa_a1_1[iscan][ifov], 100.0);

        fread(&geoline, L_GEOLINE, 1, fp_a12_geo);
        lat_a1_2[iscan][ifov] = geoline.lat*1.0E-5; 
        lon_a1_2[iscan][ifov] = geoline.lon*1.0E-5; 
        lat_a1_2[iscan][ifov] = ftrunc(lat_a1_2[iscan][ifov], 1000.0); 
        lon_a1_2[iscan][ifov] = ftrunc(lon_a1_2[iscan][ifov], 1000.0); 
        if(ifov < NUMSPOT_A/2)
        { 
          lza_a1_2[iscan][ifov] = -geoline.lza*1.0E-5; 
          ssa_a1_2[iscan][ifov] = -geoline.ssa*1.0E-5;
        } 
        else
        {
          lza_a1_2[iscan][ifov] = geoline.lza*1.0E-5; 
          ssa_a1_2[iscan][ifov] = geoline.ssa*1.0E-5; 
        }
        lza_a1_2[iscan][ifov] = ftrunc(lza_a1_2[iscan][ifov], 100.0); 
        ssa_a1_2[iscan][ifov] = ftrunc(ssa_a1_2[iscan][ifov], 100.0);

        fread(&geoline, L_GEOLINE, 1, fp_a2_geo);
        lat_a2[iscan][ifov] = geoline.lat*1.0E-5; 
        lon_a2[iscan][ifov] = geoline.lon*1.0E-5; 
        lat_a2[iscan][ifov] = ftrunc(lat_a2[iscan][ifov], 1000.0); 
        lon_a2[iscan][ifov] = ftrunc(lon_a2[iscan][ifov], 1000.0); 
        if(ifov < NUMSPOT_A/2)
        { 
          lza_a2[iscan][ifov] = -geoline.lza*1.0E-5; 
          ssa_a2[iscan][ifov] = -geoline.ssa*1.0E-5;
        } 
        else
        {
          lza_a2[iscan][ifov] = geoline.lza*1.0E-5; 
          ssa_a2[iscan][ifov] = geoline.ssa*1.0E-5; 
        }
        lza_a2[iscan][ifov] = ftrunc(lza_a2[iscan][ifov], 100.0); 
        ssa_a2[iscan][ifov] = ftrunc(ssa_a2[iscan][ifov], 100.0);
      }
/*---------------------------------------------------*
 * Print out some scanline information
 *---------------------------------------------------*/
/*     printf(" scan num    :  %d\n",geohead.nscan);
     printf(" iscan, ifov :  %d, %d\n", iscan, ifov);
     printf(" first a2 lat/lon  :  %f, %f\n", lat_a2[0][0], lon_a2[0][0]);
     printf(" first a2 lza/ssa  :  %f, %f\n", lza_a2[0][0], ssa_a2[0][0]);
     printf(" last a2 lat/lon  :  %f, %f\n",
           lat_a2[iscan-1][ifov-1], lon_a2[iscan-1][ifov-1]);
     printf(" last a2 lza/ssa  :  %f, %f\n",
           lza_a2[iscan-1][ifov-1], ssa_a2[iscan-1][ifov-1]);
     printf(" first a1_1 lat/lon  :  %f, %f\n", lat_a1_1[0][0], lon_a1_1[0][0]);
     printf(" last a1_1 lat/lon  :  %f, %f\n",
           lat_a1_1[iscan-1][ifov-1], lon_a1_1[iscan-1][ifov-1]);
*/
/*---------------------------------------------------*
 * Close geolocation files
 *---------------------------------------------------*/
   fclose(fp_a11_geo);
   fclose(fp_a12_geo);
   fclose(fp_a2_geo);

   return(0);

} /* end of rama_scn.c */

float ftrunc(float var, float scale)
//float var;
//float scale;
{
   float retvar;

   if (var < 0.0)
     retvar = (float)((int)(var * scale - 0.5)) / scale;
   else
     retvar = (float)((int)(var * scale + 0.5)) / scale;
   return(retvar);
}
