/***************************************************************************
 *  Program Name      : read_nc_time.c
 *  Type              : Main program
 *  Function          : Program reads text string of scan time, and convert it 
                        to corresponding integer values of year, month, day, 
                        hour, minute and second
 *  Input Files       : AMSU-A FCDR file
 *  Output Files      : None 
 *  Subroutine Called : None 
 *  Called by         : None 
 *************************************************************************/
#include "netcdf.h"
#include <sys/stat.h>
#include <unistd.h> 
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>

/* Handle errors by printing an error message and exiting with a
 * non-zero status. */
#define ERRCODE 2
#define ERR(e) {printf("Error: %s\n", nc_strerror(e)); exit(ERRCODE);}

#define MSL 950		// maximum scanlines
#define NCHAR 20        // number of characters in a time record

int main(int argc, char *argv[])
{
     int         status;
     int         ncid, grpid, varid, nscanid;
     int         nscanlen;
     char        fname[100];
     char        str_scantime[MSL][NCHAR];
     char        buffer[80];
     int         year[MSL], mon[MSL], day[MSL];
     int         hh[MSL], mm[MSL], ss[MSL];
     int         iscan;

   if (argc < 2)
   {
      printf("Need input nc file name!\n");
      exit(1);
   } 
   
   strcpy(fname,argv[1]);
   
   status = nc_open(fname, 0, &ncid);			// open NetCDF dataset
   if (status != NC_NOERR) ERR(status);

   status = nc_inq_dimid(ncid, "nscan", &nscanid);	// inquire dimension ID 
   if (status != NC_NOERR) ERR(status);

   status = nc_inq_dimlen(ncid, nscanid, &nscanlen);	// inquire dimension length
   if (status != NC_NOERR) ERR(status);
   printf("total %d scanlines\n", nscanlen);

   status = nc_inq_grp_ncid(ncid, "Geolocation_Time_Fields", &grpid);	// inquire group ID
   if (status != NC_NOERR) ERR(status);

   status = nc_inq_varid(grpid, "scan_time", &varid);	// inquire variable ID
   if (status != NC_NOERR) ERR(status);

   status = nc_get_var_text(grpid, varid, &str_scantime[0][0]);		// get variable
   if (status != NC_NOERR) ERR(status);

   for (iscan = 0; iscan < nscanlen; iscan ++)		// parse variable
     sscanf(str_scantime[iscan], "%4d-%2d-%2d %2d:%2d:%d",
        &year[iscan], &mon[iscan], &day[iscan],
        &hh[iscan], &mm[iscan], &ss[iscan]); 

   status = nc_close(ncid);       			// close netCDF dataset 
   if (status != NC_NOERR) ERR(status);
 
   sscanf(str_scantime[nscanlen-1],"%20c",buffer);	// display variable 
   printf("%s\n",buffer);

   iscan --;
   printf("%d %d %d %d %d %d\n", year[iscan], mon[iscan], day[iscan],	// display parse results
        hh[iscan], mm[iscan], ss[iscan]); 

}
