/***************************************************************************
 *  Program Name      : set_sw.c
 *  Type              : Subroutine
 *  Function          : Program sets up AMSU-A NETCDF swath file 
 *  Input Files       : None 
 *  Output Files      : None
 *  Subroutine Called : minmax.c 
 *  Called by         : rama_wfcdr.c
 *
 *  Modification History:
 *      Date       Version         Description of Changes
 *     ------     ---------       ------------------------
 *   7/27/2011      v1.0
 *************************************************************************/
#include "AMA2NC_INCLUDE.h"
#include "ESWATH.h"

#define NDIMS 2

/* Handle errors by printing an error message and exiting with a
 * non-zero status. */
#define ERRCODE 2
#define ERR(e) {printf("Error: %s\n", nc_strerror(e)); exit(ERRCODE);}

float *minmax();

/******************************************************************/
/*long int sw_def (char *nc_filename, short int  num_of_scan);
long int sw_wrt (char *nc_filename, short int  num_of_scan);
*/
/******************************************************************/
void set_sw(char *nc_filename, short int numscan)
{ 
   int        status;
   char       chan_name[9];

   int        ncid, x_dimid, y_dimid, c_dimid, chan_dimid, varid, grpid;
   int        dimids[NDIMS], cdimids[1], chandimids[NDIMS];
   size_t     chunks[NDIMS];
   int        shuffle, deflate, deflate_level;
   float      data_out[MAXSCANLINE_A][NUMSPOT_A];
   int        i, j, k, x, y, retval;
   float      at_scal = 1.0;
   float      at_tmp;

   time_t     rawtime;
   struct tm* timeinfo;
   char       buffer[80], buffer2[80];
   float      *latmm, *lonmm; 
   float      latv[2],lonv[2];
   double     dtime;

   float      fv = MISSING;
   double     fv_zero = 0.0;
   unsigned char fv_ubyte = 255;
   int        chsize =  MAXSCANLINE_A/5;   // 190

   char       *Rl_Vname[] = {"Rl_23","Rl_31","Rl_50","Rl_89"};
   char       *Z_Vname[] = {"Z_23","Z_31","Z_50","Z_89"};
   char       *BT_Vname[] = {"fcdr_brightness_temperature_23",
                             "fcdr_brightness_temperature_31",
                             "fcdr_brightness_temperature_50",
                             "fcdr_brightness_temperature_89"};
   char       *BT_Lname[] = {"NOAA FCDR of 23.8 GHz brightness temperature",
                             "NOAA FCDR of 31.4 GHz brightness temperature",
                             "NOAA FCDR of 50.3 GHz brightness temperature",
                             "NOAA FCDR of 89.0 GHz brightness temperature"};
   char       *coord[] = {"latitude_a2 longitude_a2",
                          "latitude_a2 longitude_a2",
                          "latitude_a1_2 longitude_a1_2",
                          "latitude_a1_1 longitude_a1_1"};
   int        len_coord[] = {24, 24, 28, 28};
   float      lat_lim[] = {-90.0, 90.0};
   float      lon_lim[] = {-180.0, 180.0};
   unsigned char flag_orb[] = {0, 1};
   char       str_orb[] = "northbound southbound";
   signed char flag_stype[] = {0, 1, 2};
   char       str_stype[] = "ocean land coast";

   /* Set chunking, shuffle, and deflate. */
   shuffle = NC_SHUFFLE;
   deflate = 1;
   deflate_level = 1;

   /* Create the file. The NC_NETCDF4 parameter tells netCDF to create
    * a file in netCDF-4/HDF5 standard. */
   if ((retval = nc_create(nc_filename, NC_CLOBBER|NC_NETCDF4, &ncid)))
      ERR(retval);

   /* Define the dimensions. */
   if ((retval = nc_def_dim(ncid, "nscan", numscan, &x_dimid)))
      ERR(retval);
   if ((retval = nc_def_dim(ncid, "npixel", NUMSPOT_A, &y_dimid)))
      ERR(retval);
   if ((retval = nc_def_dim(ncid, "nchar", NCHAR, &c_dimid)))
      ERR(retval);
   if ((retval = nc_def_dim(ncid, "nchan", NUMCHAN_A, &chan_dimid)))
      ERR(retval);

   /* Set up variabe data. */
   dimids[0] = x_dimid;
   dimids[1] = y_dimid;
   cdimids[0] = x_dimid;
   cdimids[1] = 1;
   cdimids[1] = c_dimid;
   chandimids[0] = x_dimid;
   chandimids[1] = chan_dimid;
   chunks[0] = MAXSCANLINE_A/5;
   chunks[1] = NUMSPOT_A/5;

/*--------------------------------------------------------*
 * Global attributes 
 *--------------------------------------------------------*/
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "Conventions", 
                          6, "CF-1.6")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "Metadata_Conventions", 
                          63, "CF-1.6, Unidata Dataset Discovery v1.0, NOAA CDR v1.0, GDS v2.0")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "standard_name_vocabulary", 
                          49, "CF Standard Name Table (Version 34, 13 June 2016)")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "id", 
                          sizeof(nc_fname), nc_fname)))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "naming_authority", 
                          13, "gov.noaa.ncdc")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "metadata_link", 
                          20, "gov.noaa.ncdc:C00980")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "title", 
                          25, "CICS Version-1 AMSUA FCDR")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "product_version", 
                          6, "v00r02")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "date_issued", 
                          3, "TBD")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "summary", 
                          235, "CDRs for AMSU window channels and corresponding hydrological products are vital for the climate community. The corrections applied to the CDRs include geolocation correction, AMSU-A scan bias corrections, and intersatellte calibration.")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "keywords", 
                          73, "EARTH SCIENCE > SPECTRAL/ENGINEERING > MICROWAVE > BRIGHTNESS TEMPERATURE")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "keywords_vocabulary", 
                          78, "NASA Global Change Master Directory (GCMD) Earth Science Keywords, Version 8.1")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "platform", 
                          62, platform)))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "sensor", 
                          45, "AMSU-A > Advanced Microwave Sounding Unit - A")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "cdm_data_type", 
                          5, "Swath")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "cdr_program", 
                          56, "NOAA Climate Data Record Program for satellites, FY 2016")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "cdr_variable", 
                          126, "fcdr_brightness_temperature_23, fcdr_brightness_temperature_31, fcdr_brightness_temperature_50, fcdr_brightness_temperature_89")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "source", 
                          42, hdf_fname)))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "references", 
                          135, "Original level 1b data are from NOAA archive, they are also available at NOAA Comprehensive Large Array-data Stewardship System (CLASS)")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "history", 
                          3, "TBD")))
      ERR(retval);

   time(&rawtime);
   timeinfo = gmtime(&rawtime);
   strftime(buffer,80,"%Y-%m-%dT%H:%M:%S",timeinfo);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "date_created", 
                          sizeof(buffer), buffer)))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "creator_name", 
                          26, "Huan Meng, Ralph R Ferraro")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "creator_url", 
                          38, "http://cics.umd.edu/AMSU-CDR/home.html")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "creator_email", 
                          44, "Huan.Meng@noaa.gov, Ralph.R.Ferraro@noaa.gov")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "institution", 
                          147, "DOC/NOAA/NESDIS/STAR/CoRP > Cooperative Research Program, Center for Satellite Applications and Research, NESDIS, NOAA, U.S. Department of Commerce")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "processing_level", 
                          12, "NOAA level 2")))
      ERR(retval);

   latmm = minmax(lat_a1_1,numscan,NUMSPOT_A);
   latv[0] = *latmm;
   latv[1] = *(latmm+1);
   lonmm = minmax(lon_a1_1,numscan,NUMSPOT_A);
   lonv[0] = *lonmm;
   lonv[1] = *(lonmm+1);
   if ((retval = nc_put_att_float(ncid, NC_GLOBAL, "geospatial_lat_min", NC_FLOAT,
                          1, &latv[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(ncid, NC_GLOBAL, "geospatial_lat_max", NC_FLOAT,
                          1, &latv[1])))
      ERR(retval);
   if ((retval = nc_put_att_float(ncid, NC_GLOBAL, "geospatial_lon_min", NC_FLOAT,
                          1, &lonv[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(ncid, NC_GLOBAL, "geospatial_lon_max", NC_FLOAT,
                          1, &lonv[1])))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "geospatial_lat_units", 
                          13, "degrees_north")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "geospatial_lon_units", 
                          12, "degrees_east")))
      ERR(retval);

   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "spatial_resolution", 
                          42, "48km X 48km at nadir, 150km X 80km at limb")))
      ERR(retval);

   sscanf(str_scantime[0],"%19c",buffer);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "time_coverage_start", 
                          sizeof(buffer), buffer)))
      ERR(retval);

   sscanf(str_scantime[numscan-1],"%19c",buffer);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "time_coverage_end", 
                          sizeof(buffer), buffer)))
      ERR(retval);

   dtime = time_tai93[numscan-1] - time_tai93[0];
   sprintf(buffer2,"P%.0lfS",dtime);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "time_coverage_duration", 
                          sizeof(buffer2), buffer2)))
      ERR(retval);

   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "license", 
                          32, "No restrictions on access or use")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "contributor_name", 
                          3, "TBD")))
      ERR(retval);
   if ((retval = nc_put_att_text(ncid, NC_GLOBAL, "contributor_role", 
                          3, "TBD")))
      ERR(retval);

/*--------------------------------------------------------*
 * Group 1: geolocation field 
 *--------------------------------------------------------*/
   if ((retval = nc_def_grp(ncid, "Geolocation_Time_Fields", &grpid)))
      ERR (retval);

/*--------------------------------------------------------*
 * Time fields 
 *--------------------------------------------------------*/
// scan_time
/*   if ((retval = nc_def_var(grpid, "scan_time", NC_STRING, 1,
                          &x_dimid, &varid)))
      ERR(retval);

   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          71, "Scan start time (UTC) in ISO8601 date/time (YYYY-MM-DDTHH-MM-SS) format")))
      ERR(retval);
  // if (numscan > chsize) 
  // {
  // if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
  //    ERR(retval);
  // if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
  //                                deflate_level)))
  //    ERR(retval);
  // }
   if ((retval = nc_put_var_string(grpid, varid, (const char **)str_scantime)))
      ERR(retval);
*/
// scan_time
   if ((retval = nc_def_var(grpid, "scan_time", NC_CHAR, NDIMS,
                          cdimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          71, "Scan start time (UTC) in ISO8601 date/time (YYYY-MM-DDTHH-MM-SS) format")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "_FillValue",
                          1, "0")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_text(grpid, varid, &str_scantime[0][0])))
      ERR(retval);

// scan_time_since98
   if ((retval = nc_def_var(grpid, "scan_time_since98", NC_DOUBLE, 1,
                          &x_dimid, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          4, "time")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          60, "Scan start time (UTC) in a referenced or elapsed time format")))
      ERR(retval);
   if ((retval = nc_put_att_double(grpid, varid, "_FillValue", NC_DOUBLE,
                          1, &fv_zero)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          33, "seconds since 1998-01-01T00:00:00")))
      ERR(retval);

   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_double(grpid, varid, &time_tai93[0])))
      ERR(retval);

/*--------------------------------------------------------*
 * Lat/lon fields 
 *--------------------------------------------------------*/
// latitude_a1_1
   if ((retval = nc_def_var(grpid, "latitude_a1_1", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          8, "latitude")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          43, "Latitude for AMSU unit A1-1, e.g., 89.0 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_min", NC_FLOAT,
                          1, &lat_lim[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_max", NC_FLOAT,
                          1, &lat_lim[1])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          13, "degrees_north")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lat_a1_1[0][0])))
      ERR(retval);

// latitude_a1_2
   if ((retval = nc_def_var(grpid, "latitude_a1_2", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          8, "latitude")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          43, "Latitude for AMSU unit A1-2, e.g., 53.0 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_min", NC_FLOAT,
                          1, &lat_lim[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_max", NC_FLOAT,
                          1, &lat_lim[1])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          13, "degrees_north")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lat_a1_2[0][0])))
      ERR(retval);

// latitude_a2
   if ((retval = nc_def_var(grpid, "latitude_a2", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          8, "latitude")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          58, "Latitude for AMSU unit A2, including 23.8 GHz and 31.4 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_min", NC_FLOAT,
                          1, &lat_lim[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_max", NC_FLOAT,
                          1, &lat_lim[1])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          13, "degrees_north")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lat_a2[0][0])))
      ERR(retval);

// longitude_a1_1
   if ((retval = nc_def_var(grpid, "longitude_a1_1", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          9, "longitude")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          44, "Longitude for AMSU unit A1-1, e.g., 89.0 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_min", NC_FLOAT,
                          1, &lon_lim[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_max", NC_FLOAT,
                          1, &lon_lim[1])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          13, "degrees_east")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lon_a1_1[0][0])))
      ERR(retval);

// longitude_a1_2
   if ((retval = nc_def_var(grpid, "longitude_a1_2", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          9, "longitude")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          44, "Longitude for AMSU unit A1-2, e.g. 50.3 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_min", NC_FLOAT,
                          1, &lon_lim[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_max", NC_FLOAT,
                          1, &lon_lim[1])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          13, "degrees_east")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lon_a1_2[0][0])))
      ERR(retval);

// longitude_a2
   if ((retval = nc_def_var(grpid, "longitude_a2", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          9, "longitude")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          59, "Longitude for AMSU unit A2, including 23.8 GHz and 31.4 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_min", NC_FLOAT,
                          1, &lon_lim[0])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "valid_max", NC_FLOAT,
                          1, &lon_lim[1])))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          13, "degrees_east")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lon_a2[0][0])))
      ERR(retval);

/*--------------------------------------------------------*
 * Group 2: data field 
 *--------------------------------------------------------*/
   if ((retval = nc_def_grp(ncid, "Data_Fields", &grpid)))
      ERR (retval);
 
/*--------------------------------------------------------*
 * Orbit mode data field 
 *--------------------------------------------------------*/
   if ((retval = nc_def_var(grpid, "orbital_mode", NC_UBYTE, 1,
                          &x_dimid, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          19, "satellite direction")))
      ERR(retval);
   if ((retval = nc_put_att_uchar(grpid, varid, "flag_values", NC_UBYTE,
                          2, &flag_orb[0])))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "flag_meanings", 
                          sizeof(str_orb), str_orb)))
      ERR(retval);
   if ((retval = nc_put_att_uchar(grpid, varid, "_FillValue", NC_UBYTE,
                          1, &fv_ubyte)))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_uchar(grpid, varid, &orb_mode[0])))
      ERR(retval);

/*--------------------------------------------------------*
 * Surface type data field 
 *--------------------------------------------------------*/
   if ((retval = nc_def_var(grpid, "surface_type_a1_1", NC_BYTE, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          47, "surface type for AMSU unit A1-1, e.g., 89.0 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_schar(grpid, varid, "flag_values", NC_BYTE,
                          3, &flag_stype[0])))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "flag_meanings", 
                          sizeof(str_stype), str_stype)))
      ERR(retval);
   if ((retval = nc_put_var_schar(grpid, varid, &stype_a1_1[0][0])))
      ERR(retval);

   if ((retval = nc_def_var(grpid, "surface_type_a1_2", NC_BYTE, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          47, "surface type for AMSU unit A1-2, e.g., 50.3 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_schar(grpid, varid, "flag_values", NC_BYTE,
                          3, &flag_stype[0])))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "flag_meanings", 
                          sizeof(str_stype), str_stype)))
      ERR(retval);
   if ((retval = nc_put_var_schar(grpid, varid, &stype_a1_2[0][0])))
      ERR(retval);

   if ((retval = nc_def_var(grpid, "surface_type_a2", NC_BYTE, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          62, "surface type for AMSU unit A2, including 23.8 GHz and 31.4 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_schar(grpid, varid, "flag_values", NC_BYTE,
                          3, &flag_stype[0])))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "flag_meanings", 
                          sizeof(str_stype), str_stype)))
      ERR(retval);
   if ((retval = nc_put_var_schar(grpid, varid, &stype_a2[0][0])))
      ERR(retval);


/*--------------------------------------------------------*
 * Angle data fields 
 *--------------------------------------------------------*/
// earth_incidence_angle_a1_1
   if ((retval = nc_def_var(grpid, "earth_angle_of_incidence_a1_1", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          59, "earth angle of incidence for AMSU unit A1-1, e.g., 89.0 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          6, "degree")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "coordinates", 
                          len_coord[3], coord[3])))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lza_a1_1[0][0])))
      ERR(retval);

// earth_incidence_angle_a1_2
   if ((retval = nc_def_var(grpid, "earth_angle_of_incidence_a1_2", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          59, "earth angle of incidence for AMSU unit A1-2, e.g., 50.3 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          6, "degree")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "coordinates", 
                          len_coord[2], coord[2])))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lza_a1_2[0][0])))
      ERR(retval);

// earth_incidence_angle_a2
   if ((retval = nc_def_var(grpid, "earth_angle_of_incidence_a2", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          74, "earth angle of incidence for AMSU unit A2, including 23.8 GHz and 31.4 GHz")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          6, "degree")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "coordinates", 
                          len_coord[0], coord[0])))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &lza_a2[0][0])))
      ERR(retval);

// solar_zenith_angle
   if ((retval = nc_def_var(grpid, "solar_zenith_angle", NC_FLOAT, NDIMS,
                          dimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                          18, "solar zenith angle")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          39, "solar zenith angle for all AMSU-A units")))
      ERR(retval);
   if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                          1, &fv)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "units", 
                          6, "degree")))
      ERR(retval);
   if (numscan > chsize) 
   {
   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
      ERR(retval);
   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                  deflate_level)))
      ERR(retval);
   }
   if ((retval = nc_put_var_float(grpid, varid, &sza[0][0])))
      ERR(retval);

/*--------------------------------------------------------*
 * AT data fields
 *--------------------------------------------------------*/
   for(k = 0; k < NUMCHAN_A; k++)
   {
     if ((retval = nc_def_var(grpid, BT_Vname[k], NC_FLOAT, NDIMS,
                            dimids, &varid)))
        ERR(retval);

   /* Write attribute data. */
     if ((retval = nc_put_att_text(grpid, varid, "standard_name", 
                            22, "brightness_temperature")))
        ERR(retval);
     if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                            44, BT_Lname[k])))
        ERR(retval);
     if ((retval = nc_put_att_float(grpid, varid, "valid_min", NC_FLOAT,
                            1, &limit_A.Temp_lower[k])))
        ERR(retval);
     if ((retval = nc_put_att_float(grpid, varid, "valid_max", NC_FLOAT,
                            1, &limit_A.Temp_upper[k])))
        ERR(retval);
     if ((retval = nc_put_att_float(grpid, varid, "_FillValue", NC_FLOAT,
                            1, &fv)))
        ERR(retval);
     if ((retval = nc_put_att_text(grpid, varid, "units", 
                            6, "kelvin")))
        ERR(retval);
     if ((retval = nc_put_att_text(grpid, varid, "coordinates", 
                            len_coord[k], coord[k])))
        ERR(retval);

   /* Chunking and deflate. */
   if (numscan > chsize) 
   {
     if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
        ERR(retval);
     if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
                                    deflate_level)))
        ERR(retval);
   }

   /* Applying asym correction to calculated BT data. */     
     for(i = 0; i < numscan; i++)
        for(j = 0; j < NUMSPOT_A; j++)
        {
          if(at[i][j][k] != MISSING)
          {
             at_tmp = at[i][j][k] * (1 + asym_a1[j][k]) + asym_a0[j][k];
             at_tmp = at_tmp + asym_a2[j][k] * at[i][j][k] * at[i][j][k];
//             data_out[i][j] = ftrunc(at_tmp, 100.0);
             data_out[i][j] = (float)((int)(at_tmp*100.0 + 0.5))/100.0;
//             data_out[i][j] = (float)((int)((at[i][j][k] + asymdata[j][k])*100))/100;
          }
          else
             {
              data_out[i][j] = fv; //MISSING;
              qa_prod[i][k] |= 1<<7;
//              printf("%f  ", data_out[i][j]);
             }
        }
 
   /* No need to explicitly end define mode for netCDF-4 files. Write
    * the calculated data to the file. */
   if ((retval = nc_put_var_float(grpid, varid, &data_out[0][0])))
      ERR(retval);
   }

/*--------------------------------------------------------*
 * Product Quality Flag data field 
 *--------------------------------------------------------*/
   if ((retval = nc_def_var(grpid, "flag_brightness_temperature", NC_UBYTE, NDIMS,
                          chandimids, &varid)))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "long_name", 
                          31, "instrument/channel quality flag")))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "comment", 
                          581, "The 8-bit quality flags are channel-specific. Users are advised not to use any scans for which the highest bit (bit 7) is set to 1. The lowest two bits (bits 0 and 1) are not used. The meanings of the bits from the MSB (bit 7) to the LSB (bit 0), if set to 1, are: bit 7 = Do not use scan for product generation; bit 6 = Calibration error; bit 5 = Time sequence error; bit 4 = Earth location questionable; bit 3 = Brightness temperature out of range; bit 2 = Lunar contamination warning; bit 1~0 = zero fill. If any bit is set to 0, then there is no correspondent error or warning.")))
      ERR(retval);
   if ((retval = nc_put_att_uchar(grpid, varid, "flag_masks", NC_UBYTE,
                          2, &flag_orb[0])))
      ERR(retval);
   if ((retval = nc_put_att_text(grpid, varid, "flag_meanings", 
                          16, "good problematic")))
      ERR(retval);
//   if ((retval = nc_def_var_chunking(grpid, varid, 0, &chunks[0])))
//      ERR(retval);
//   if ((retval = nc_def_var_deflate(grpid, varid, shuffle, deflate,
//                                  deflate_level)))
//      ERR(retval);
   if ((retval = nc_put_var_uchar(grpid, varid, &qa_prod[0][0])))
      ERR(retval);



   /* Close the file. */
   if ((retval = nc_close(ncid)))
      ERR(retval);

   free(*stype_a1_1);
   free(stype_a1_1);
   free(*stype_a1_2);
   free(stype_a1_2);
   free(*stype_a2);
   free(stype_a2);

   printf("*** SUCCESS writing example file %s!\n", nc_filename);
}/*  end of set_sw.c  */
