#!/bin/bash
# 
# Function: Main script to generate AMSU-A FCDR for window channels.
#
# Usage: time.bash
#
# Note 1: User need to organize AMSU-A L1b and Geolocation data beforehand, 
# and specify their root paths, e.g., base_l1b and base_geo. 
# The root path for output FCDR, base_des is also needed.
# 
# Note 2: There are two for loops to specify satellite and year.
#
# To generate only one satellite, e.g., noaa-16, use
# for sat in noaa-16
# in the outer for loop; to generate all satellites, use 
# for sat in `ls $base_l1b` 
# in the outer for loop;
#
# To generate only two years, e.g., 2009 and 2010, use
# for year in 2009 2010
# in the inner for loop; to generate all years, use
# for year in `ls $sat_dir` 
# in the inner for loop.

base_l1b=/amsu/amsu/1b/amsu-a
base_geo=/amsu/amsu/1b-geo/amsu-a
base_des=/datar/wyang/data/beta_data

#for sat in noaa-16
for sat in `ls $base_l1b`
do
  echo $sat
  sat_dir=$base_l1b/$sat
#  for year in 2009 2010
  for year in `ls $sat_dir`
  do
    echo $year
    l1b_dir=$sat_dir/$year/
    ls $l1b_dir | grep AMAX > amafile
    geo_dir=$base_geo/$sat/$year/
    out_dir=$base_des/$sat/$year/
    time ./runama $l1b_dir $geo_dir $out_dir > ama2nc.log 2> ama2nc.error
  done
done

