/*************************************************************
 * File name   : AMA1B.h
 * Function    : Header and data structures for AMSU-A 1B
 * Update      : 05/08/2006
 *************************************************************/

/*---------------------------------*
 *         header record           *
 *---------------------------------*/

typedef struct
{
  char			dummy0[72];
  unsigned short	sat_id;
  char                  dummy1[10];
  unsigned short        start_year;
  unsigned short        start_day_of_year;
  unsigned int          start_milliseconds_of_day;
  char                  dummy2[4];
  unsigned short        end_year;
  unsigned short        end_day_of_year;
  unsigned int          end_milliseconds_of_day;
  char                  dummy3[40];
  unsigned short        last_scan_record;
//  char                  dummy4[542];
  char			dummy4[70];
  short                 rfs[3][3];
  char			dummy41[30];
  short			corr[15][4];
//  char		  dummy42[304];
  char			dummy42[36];
  int			mu[15][3];
  char			dummy43[88];
  int                   t_r_central_wave_number_1;
  char                  dummy5[8];
  int                   t_r_central_wave_number_2;
  char                  dummy6[8];
  int                   t_r_central_wave_number_3;
  char                  dummy7[8];
  int                   t_r_central_wave_number_4;
  char                  dummy8[8];
  int                   t_r_central_wave_number_5;
  char                  dummy9[8];
  int                   t_r_central_wave_number_6;
  char                  dummy10[8];
  int                   t_r_central_wave_number_7; 
  char                  dummy11[8];
  int                   t_r_central_wave_number_8;
  char                  dummy12[8];
  int                   t_r_central_wave_number_9; 
  char                  dummy13[8];
  int                   t_r_central_wave_number_10; 
  char                  dummy14[8];
  int                   t_r_central_wave_number_11;
  char                  dummy15[8];
  int                   t_r_central_wave_number_12;
  char                  dummy16[8];
  int                   t_r_central_wave_number_13;
  char                  dummy17[8];
  int                   t_r_central_wave_number_14;
  char                  dummy18[8];
  int                   t_r_central_wave_number_15;
  char                  dummy19[40];
  unsigned short        orbit_vector_epoch_year;
  unsigned short        orbit_vector_day_of_year;
  unsigned int          orbit_vector_utc_time_of_day;
  int                   semimajor_axis;
  int                   eccentricity;
  int                   inclination;
  int                   argument_of_perigee;
  int                   right_ascension;
  int                   mean_anomaly;
  int                   posision_x;
  int                   posision_y;
  int                   posision_z;
  int                   velocity_x;
  int                   velocity_y;
  int                   velocity_z;
  unsigned int          eart_sun_dist_ratio;
//  char                  dummy20[1600];
  char			dummy20[528];
  int			rfs_coeffs_a1[2][4];
  char			dummy21[16];
  int			warm_digital_coeffs_a1[10][4];
  char			dummy22[384];
  int			rfs_coeffs_a2[4];
  char			dummy23[16];
  int			warm_digital_coeffs_a2[7][4];
  char			dummy24[336];

}HBLOCK;
#define L_HBLOCK  sizeof(HBLOCK)

/*---------------------------------*
 *         scanline data           *
 *---------------------------------*/

typedef struct
{
  char                  dummy1[8];
  unsigned int          time_of_day_of_scan;
  unsigned short        orbit_node; 
  char                  dummy2[10];
  unsigned int          quality_indicator_bit_field;
  char                  dummy3;
  unsigned char         qa_time;
  unsigned char         qa_cal;
  unsigned char         qa_eloc;
  unsigned short        qa_cal_chan[15];
  char                  dummy31[18];
  int                   pri_cal_coeffs[15][3];
  char                  dummy4[212];
  short                 angular_relationships[90];
  int                   earth_location[60];
  char                  dummy5[12];
  unsigned short        scene_telemetry_a1[510];
  unsigned short	cold_cal_telemetry_a1[30];
//  char                  dummy61[92];
  char			dummy61[64];
  unsigned short	count_rfs_a1[2];
  char			dummy62[2];
  unsigned short   	countwd_a1[10];
  char			dummy63[2];
  unsigned short	warm_cal_telemetry_a1[30];
  char			dummy6[56];
  unsigned short        scene_telemetry_a2[120];
  unsigned short	cold_cal_telemetry_a2[6];
//  char                  dummy71[40];
  char			dummy71[20];
  unsigned short	count_rfs_a2;
  char			dummy72[2];
  unsigned short   	countwd_a2[7];
  char			dummy73[2];  
  unsigned short	warm_cal_telemetry_a2[6];
//  char                  dummy7[64];
  char                  dummy7[32];
  unsigned char		countc_corr[15];
  char			dummy74[17];

}SCANLINE;
#define L_SCANLINE  sizeof(SCANLINE)

/*---------------------------------*
 *         geo head           *
 *---------------------------------*/

typedef struct
{
  int                   pitch;
  int                   roll;
  int                   yaw;
  int                   nscan;

}GEOHEAD;
#define L_GEOHEAD  sizeof(GEOHEAD)

/*---------------------------------*
 *         geo data           *
 *---------------------------------*/

typedef struct
{
  int                   lat;
  int                   lon;
  int                   lza;
  int                   ssa;

}GEOLINE;
#define L_GEOLINE  sizeof(GEOLINE)

